/*
 * FILE --- lqr_ctrl.hh
 *
 * PURPOSE --- Header file for a class that implements a State Feddback or
 * Linear Quadratic Regulator (LQR) controller. The user specifies the
 * dimension of the state of the system under control and the LQR gains.
 *
 */

// Prevent Multiple Inclusion
#ifndef LQR_CTRL_HH
#define LQR_CTRL_HH

// Include Files
#include <stdlib.h>

// Local Definitions
#define LQR_CTRL_NO_ERROR 0
#define LQR_CTRL_MEM_ERROR 1

class LQR_CTRL
{
public:

  // Constructor
  LQR_CTRL(int _plant_order);

  // Destructor
  ~LQR_CTRL();

  // The main control routine to calculate the controller output 
  // _input = an array of size plant_order containing the current state of
  // the system under control or basically the input to the LQR 
  double control(const double *_input);

  // Set the gains of the LQR controller
  // _lqr_gains = an array of size plant_order specifying the feedback gains
  // if _lqr_gains == NULL then all the gains are initialized to zero
  void set_gains(double *_lqr_gains = NULL);

  int lqr_ctrl_err;  // Variable to hold the error condition

private:
  // Allocate/free memory for the gains
  void allocate_memory(void);
  void free_memory(void);

  int plant_order;  // The dimension of the state of the controlled system 
  double *lqr_gains; // An array to hold LQR or feedback gains
};

#endif // LQR_CTRL_HH
