/*
 * FILE --- my_observer.cc
 *
 * PURPOSE --- Source file for a class that provides an example of usage
 * of the Linear State Observer utility for Single-Input-Single-Output (SISO) 
 * plants.
 *
 */

// Include Files
#include "my_observer.hh"

MY_OBSERVER::MY_OBSERVER(int _plant_order, double _sample_time = 0.01)
  : OBSERVER(_plant_order, _sample_time)
{
  // Initialize the plant parameters of the system to be observer
  a11 =  2.0; a12 = 1.0;
  a21 = -1.0; a22 = 1.0;
  
  b1 = 1.0;
  b2 = 2.0;
  
  c1 = 1.0; c2 = 1.0;
  
  output_error = 0.0;
}

MY_OBSERVER::~MY_OBSERVER()
{
  
}

void MY_OBSERVER::observe(const double _input_to_plant, 
			  const double _output_of_plant)
{
  // Assign the values for the simulator
  ctrl_input[0] = _input_to_plant; 
  ctrl_input[1] = _output_of_plant; 
  
  // Simulate the observer, i.e., calculate the observer state
  for (int ii=0; ii < sim_per_sample; ii++)
    {
      solve();
    }
}

inline void MY_OBSERVER::DERIV(const double t, const double *x, 
			       const double *u, double *xdot)
{
  // Implement your system in the form
  // xhatdot = A*xhat + B*u[0] + L*(u[1] - C*xhat);
  // where u[0] is the control input to the original system and
  // u[1] is the output of the actual system
  // i.e., u[1] = y = C*x

  // Calculate the output error
  output_error = u[1] - c1*x[0] - c2*x[1];

  // Reproduce the plant dynamics
  xdot[0] = a11*x[0] + a12*x[1] + b1*u[0] + get_gain(0)*output_error;
  xdot[1] = a21*x[0] + a22*x[1] + b2*u[0] + get_gain(1)*output_error;
}
